"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import errno
import glob
import json
import sys
import shutil

try:
    QT_VERSION = 4
    from PyQt4.QtGui import QMessageBox, QFont
except ImportError:
    QT_VERSION = 5
    from PyQt5.QtWidgets import QMessageBox
    from PyQt5.QtGui import QFont

from transphire.separator import Separator
from transphire.mountcontainer import MountContainer
from transphire.statuscontainer import StatusContainer
from transphire.settingscontainer import SettingsContainer
from transphire.framecontainer import FrameContainer
from transphire.buttoncontainer import ButtonContainer
from transphire.notificationcontainer import NotificationContainer
from transphire.plotcontainer import PlotContainer
from transphire.tabdocker import TabDocker
from transphire import transphire_content as tc
from transphire import transphire_plot as tp
from transphire import transphire_import as ti


def copy(file_in, file_out):
    """
    Copy file_in to a new location.

    Arguments:
    file_in - Input file
    file_out - Output file

    Return:
    None
    """

    try:
        shutil.copy2(file_in, file_out)
    except PermissionError:
        shutil.copyfile(file_in, file_out)


def get_function_dict():
    """
    Return a dictionary containing the function to use for specific plots.

    Arguments:
    None

    Return:
    None
    """
    function_dict = {
        'CTFFIND4 v4.1.10': {
            'plot': tp.update_ctffind_4_v4_1_10,
            'plot_data': ti.import_ctffind_v4_1_10,
            'content': tc.default_ctffind_4_v4_1_10,
            'executable': True,
            'typ': 'ctf',
            'allow_empty': ['Gain file'],
            },
        'CTFFIND4 v4.1.8': {
            'plot': tp.update_ctffind_4_v4_1_8,
            'plot_data': ti.import_ctffind_v4_1_8,
            'content': tc.default_ctffind_4_v4_1_8,
            'executable': True,
            'typ': 'ctf',
            'allow_empty': ['Gain file'],
            },
        'Gctf v1.18': {
            'plot': tp.update_gctf_v1_18,
            'plot_data': ti.import_gctf_v1_18,
            'content': tc.default_gctf_v1_18,
            'executable': True,
            'typ': 'ctf',
            'allow_empty': [],
            },
        'Gctf v1.06': {
            'plot': tp.update_gctf_v1_06,
            'plot_data': ti.import_gctf_v1_06,
            'content': tc.default_gctf_v1_06,
            'executable': True,
            'typ': 'ctf',
            'allow_empty': [],
            },
        'CTER v1.0': {
            'plot': tp.update_cter_v1_0,
            'plot_data': ti.import_cter_v1_0,
            'content': tc.default_cter_v1_0,
            'executable': True,
            'typ': 'ctf',
            'allow_empty': [],
            },
        'MotionCor2 v1.0.0': {
            'plot': tp.update_motion_cor_2_v1_0_0,
            'plot_data': ti.import_motion_cor_2_v1_0_0,
            'content': tc.default_motion_cor_2_v1_0_0,
            'executable': True,
            'typ': 'motion',
            'allow_empty': ['-DefectFile', '-Gain'],
            },
        'MotionCor2 v1.0.5': {
            'plot': tp.update_motion_cor_2_v1_0_5,
            'plot_data': ti.import_motion_cor_2_v1_0_5,
            'content': tc.default_motion_cor_2_v1_0_5,
            'executable': True,
            'typ': 'motion',
            'allow_empty': ['-Dark', '-DefectFile', '-Gain'],
            },
        'MotionCor2 v1.1.0': {
            'plot': tp.update_motion_cor_2_v1_1_0,
            'plot_data': ti.import_motion_cor_2_v1_1_0,
            'content': tc.default_motion_cor_2_v1_1_0,
            'executable': True,
            'typ': 'motion',
            'allow_empty': ['-Dark', '-DefectFile', '-Gain'],
            },
        'Mount': {
            'plot': None,
            'plot_data': None,
            'content': tc.default_mount,
            'executable': False,
            'typ': None,
            'allow_empty': [],
            },
        'Pipeline': {
            'plot': None,
            'plot_data': None,
            'content': tc.default_pipeline,
            'executable': False,
            'typ': None,
            'allow_empty': [],
            },
        'General': {
            'plot': None,
            'plot_data': None,
            'content': tc.default_general,
            'executable': False,
            'typ': None,
            'allow_empty': [],
            },
        'Notification': {
            'plot': None,
            'plot_data': None,
            'content': tc.default_notification,
            'executable': False,
            'typ': None,
            'allow_empty': [],
            },
        'Others': {
            'plot': None,
            'plot_data': None,
            'content': tc.default_others,
            'executable': False,
            'typ': None,
            'allow_empty': [],
            },
        'Font': {
            'plot': None,
            'plot_data': None,
            'content': tc.default_font,
            'executable': False,
            'typ': None,
            'allow_empty': [],
            },
        'Copy': {
            'plot': None,
            'plot_data': None,
            'content': tc.default_copy,
            'executable': False,
            'typ': None,
            'allow_empty': [],
            },
        'Path': {
            'plot': None,
            'plot_data': None,
            'content': tc.default_path,
            'executable': False,
            'typ': None,
            'allow_empty': [],
            },
        }
    return function_dict


def mkdir_p(path):
    """
    Create output directories recursively.

    Arguments:
    path - Directory path to create

    Return:
    None
    """
    try:
        os.makedirs(path)
    except OSError as exc:
        if exc.errno == errno.EEXIST and os.path.lexists(path):
            pass
        else:
            raise


def message(text):
    """
    Show a text in a message box.

    Arguments:
    text - Text shown in the message box

    Return:
    None
    """
    dialog = QMessageBox()
    dialog.setText(text)
    dialog.exec_()


def question(head, text, parent):
    """
    Show a questions message box dialog.

    Arguments:
    head - Header of the window
    text - Text with the questions

    Return:
    True if No, False if Yes
    """
    if QT_VERSION == 4:
        result = QMessageBox.question(
            parent,
            head,
            text,
            'Yes',
            'No'
            )
    elif QT_VERSION == 5:
        result = QMessageBox.question(
            parent,
            head,
            text
            )
        if result == 16384:
            result = False
        else:
            result = True
    else:
        raise ImportError('QT version unknown! Please contact the transphire authors!')
    return result


def get_exclude_set(content):
    """
    Check the widget_2 variable, if the program should be loaded or not.

    Argument:
    content - Content as dictionary.

    Return:
    List of names to exclude
    """
    exclude_list = []
    content_with_content = []
    for entry in content:
        try:
            content_with_content.append(entry['content'])
        except KeyError:
            continue

    for entry in content_with_content:
        if not isinstance(entry, list):
            continue
        else:
            pass
        for sub_content in entry:
            for entry2 in sub_content:
                for key in entry2:
                    widget_2 = entry2[key][1]['widget_2']
                    if widget_2 is None:
                        continue
                    elif widget_2 == 'True':
                        continue
                    elif widget_2 == 'False':
                        exclude_list.append(key)
                        exclude_list.append('Plot {0}'.format(key))
                    else:
                        print(
                            'TransphireUtils: widget_2 content unknown! Exit here!',
                            widget_2
                            )
                        sys.exit(1)
    return set(exclude_list)


def reduce_programs(exclude_set=None):
    """
    Reduce the number of programs to the users preferences.

    Arguments:
    exclude_set - Set of names to not consider

    Return:
    List of content for motion, List of content for ctf
    """
    if exclude_set is None:
        exclude_set = set([])
    else:
        exclude_set = exclude_set
    content_motion = []
    content_ctf = []
    function_dict = get_function_dict()

    for key in function_dict:
        if key in exclude_set:
            continue
        elif function_dict[key]['executable']:
            if function_dict[key]['typ'] == 'motion':
                content_motion.append(key)
            elif function_dict[key]['typ'] == 'ctf':
                content_ctf.append(key)
            elif function_dict[key]['typ'] is None:
                print(key, 'is executable, but does not have a typ! Exit here!')
                sys.exit(1)
            else:
                print(key, 'has an unknown typ:', function_dict[key]['typ'], '! Exit here!')
                sys.exit(1)
        else:
            pass

    return sorted(content_motion), sorted(content_ctf)


def reduce_copy_entries(exclude_set, content):
    """
    Reduce the number of options based on the exclude_set

    Arguments:
    exclude_set - Set of names to not consider
    content - Content of the widgets

    Return:
    None
    """
    for item in content:
        for sub_item in item:
            if 'Motion' in sub_item:
                name = 'Motion'
            elif 'CTF' in sub_item:
                name = 'CTF'
            else:
                continue

            values = sub_item[name][1]['values']
            for key in exclude_set:
                if key in values:
                    values.remove(key)


def reduce_path_widget(exclude_set, content):
    """
    Reduce the number of paths based on the exclude_set

    Arguments:
    exclude_set - Set of names to not consider
    content - Content of the widgets

    Return:
    None
    """
    for item in content:
        for sub_item in item:
            keys_to_delete = []
            for key in sub_item:
                if key in exclude_set:
                    keys_to_delete.append(key)
            for key in keys_to_delete:
                del sub_item[key]


def get_key_names(settings_folder, name):
    """
    Extract mount names from related settings file.

    Arguments:
    None

    Return:
    List of mount names
    """
    default_file = '{0}/content_{1}.txt'.format(settings_folder, name)
    try:
        with open(default_file, 'r') as file_r:
            data = json.load(file_r)
    except FileNotFoundError:
        data = []

    return_dict = {
        'Copy_work': [],
        'Copy_backup': [],
        'Copy_hdd': [],
        'Copy': [],
        }
    for entry in data:
        name = None
        typ = None
        for widget in entry:
            for key in widget:
                if key == 'Mount name':
                    name = widget[key][0]
                elif key == 'Typ':
                    typ = widget[key][0]
                else:
                    pass
        return_dict[typ].append(name)
    return return_dict


def get_content_gui(content):
    """
    Create content lists to load the GUI.

    Arguments:
    content - Content as dictionary.

    Return:
    Content as list
    """

    content_motion, content_ctf = reduce_programs()

    gui_content = [
        {
            'name': 'Mount',
            'widget': MountContainer,
            'content_mount': content['Mount'],
            'layout': 'h2',
            },
        {
            'name': 'Stretch',
            'layout': 'h2'
            },
        {
            'name': 'Notification',
            'widget': NotificationContainer,
            'content': content['Notification'],
            'layout': 'h2',
            },
        {
            'name': 'Stretch',
            'layout': 'h2'
            },
        {
            'name': 'Button',
            'widget': ButtonContainer,
            'layout': 'h2',
            },
        {
            'name': 'TAB1',
            'widget': TabDocker,
            'layout': 'h3',
            },
        {
            'name': 'General',
            'widget': SettingsContainer,
            'content': content['General'],
            'layout': 'TAB1',
            'max_widgets': 12
            },
        {
            'name': 'Copy',
            'widget': SettingsContainer,
            'content': content['Copy'],
            'layout': 'TAB1',
            'max_widgets': 12
            },
        {
            'name': 'Path',
            'widget': SettingsContainer,
            'content': content['Path'],
            'layout': 'TAB1',
            'max_widgets': 12
            },
        {
            'name': 'Motion',
            'widget': TabDocker,
            'layout': 'TAB1',
            },
        {
            'name': 'CTF',
            'widget': TabDocker,
            'layout': 'TAB1',
            },
        {
            'name': 'Separator',
            'layout': 'h1',
            'separator': Separator(typ='vertical', color='black')
            },
        {
            'name': 'Status',
            'widget': StatusContainer,
            'content': content['Others'],
            'content_mount': content['Mount'],
            'content_pipeline': content['Pipeline'],
            'layout': 'h1',
            },
        {
            'name': 'Plot Motion',
            'widget': TabDocker,
            'layout': 'TAB1',
            },
        {
            'name': 'Plot CTF',
            'widget': TabDocker,
            'layout': 'TAB1',
            },
        ]

    for motion_content in content_motion:
        gui_content.append({
            'name': motion_content,
            'widget': SettingsContainer,
            'content': content[motion_content],
            'layout': 'Motion',
            'max_widgets': 12
            })
        gui_content.append({
            'name': 'Plot {0}'.format(motion_content),
            'widget': TabDocker,
            'layout': 'Plot Motion',
            })
        gui_content.append({
            'name': 'Plot per micrograph',
            'widget': PlotContainer,
            'content': 'values',
            'layout': 'Plot {0}'.format(motion_content),
            })
        gui_content.append({
            'name': 'Plot histogram',
            'widget': PlotContainer,
            'content': 'histogram',
            'layout': 'Plot {0}'.format(motion_content),
            })
    gui_content.append({
        'name': 'Frames',
        'widget': FrameContainer,
        'layout': 'Motion',
        'max_widgets': 12
        })

    for ctf_content in content_ctf:
        gui_content.append({
            'name': ctf_content,
            'widget': SettingsContainer,
            'content': content[ctf_content],
            'layout': 'CTF',
            'max_widgets': 12
            })
        gui_content.append({
            'name': 'Plot {0}'.format(ctf_content),
            'widget': TabDocker,
            'layout': 'Plot CTF',
            })
        gui_content.append({
            'name': 'Plot per micrograph',
            'widget': PlotContainer,
            'content': 'values',
            'layout': 'Plot {0}'.format(ctf_content),
            })
        gui_content.append({
            'name': 'Plot histogram',
            'widget': PlotContainer,
            'content': 'histogram',
            'layout': 'Plot {0}'.format(ctf_content),
            })

    return gui_content


def look_and_feel_small(app, font=None):
    """
    Look and feel for the default settings dialog.

    Arguments:
    app - QApplication.
    font - User provided font size (default None)

    Return:
    Style sheet
    """
    if font is None:
        font = 15.0
    else:
        font = float(font)
    app.setFont(QFont('Verdana', font))
    style_widgets = """
    QWidget { background-color: lightgrey }
    QTabWidget { background-color: lightgrey }
    QTabBar { background-color: lightgrey }
    QLineEdit { background-color: white }
    QComboBox { background-color: white }
    QPushButton {
        background-color: qradialgradient(cx:0.5, cy:0.5, fx:0.5, fy:0.5, radius:1, stop:0 white, stop:1 pink);
        border-width: 1px;
        border-style:inset;
        padding: 1px;
        border-radius: 5px
        }
    QPushButton:pressed {
        background-color: qradialgradient(cx:0.5, cy:0.5, fx:0.5, fy:0.5, radius:1, stop:0 white, stop:1 pink);
        border-width: 1px;
        border-style:outset;
        padding: 1px;
        border-radius: 5px
        }
    """
    return style_widgets


def look_and_feel(app, font=None, adjust_width=None, adjust_height=None, default=None):
    """
    Look and feel.

    Arguments:
    app - QApplication.
    font - User provided font size (default None)
    adjust_width - User provided width adjustment (default None)
    adjust_height - User provided height adjustment (default None)
    default - Default values (default None)

    Return:
    Style sheet
    """
    if font is not None:
        font = float(font)
    else:
        font = float(default[0][0]['Font'][0])
    if adjust_width is not None:
        adjust_width = float(adjust_width)
    else:
        adjust_width = float(default[0][1]['Width adjustment'][0])
    if adjust_height is not None:
        adjust_height = float(adjust_height)
    else:
        adjust_height = float(default[0][2]['Height adjustment'][0])

    start_button_width = float(default[0][3]['Start button'][0])
    notification_edit_width = float(default[0][4]['Notification edit'][0])
    notification_check_width = float(default[0][5]['Notification check'][0])
    notification_button_width = float(default[0][6]['Notification button'][0])
    mount_button_width = float(default[0][7]['Mount button'][0])
    frame_entry_width = float(default[0][8]['Frame entry'][0])
    frame_button_width = float(default[0][9]['Frame button'][0])
    frame_label_width = float(default[0][10]['Frame label'][0])
    setting_widget_width = float(default[0][11]['Setting widget'][0])
    status_name_width = float(default[0][12]['Status name'][0])
    status_info_width = float(default[0][13]['Status info'][0])
    status_quota_width = float(default[0][14]['Status quota'][0])
    widget_height = float(default[0][15]['Widget height'][0])

    app.setFont(QFont('Verdana', font))
    start_button_width = '{0}px'.format(font * start_button_width * adjust_width)
    notification_edit_width = '{0}px'.format(font * notification_edit_width * adjust_width)
    notification_check_width = '{0}px'.format(font * notification_check_width * adjust_width)
    notification_button_width = '{0}px'.format(font * notification_button_width * adjust_width)
    mount_button_width = '{0}px'.format(font * mount_button_width * adjust_width)
    frame_entry_width = '{0}px'.format(font * frame_entry_width * adjust_width)
    frame_button_width = '{0}px'.format(font * frame_button_width * adjust_width)
    frame_label_width = '{0}px'.format(font * frame_label_width * adjust_width)
    setting_widget_width = '{0}px'.format(font * setting_widget_width * adjust_width)
    status_name_width = '{0}px'.format(font * status_name_width * adjust_width)
    status_info_width = '{0}px'.format(font * status_info_width * adjust_width)
    status_quota_width = '{0}px'.format(font * status_quota_width * adjust_width)
    widget_height = '{0}px'.format(font * widget_height * adjust_height)

    # Style sheet
    style_widgets = """
    QWidget#central {{
        background-color: qradialgradient(
            cx:0,
            cy:0,
            fx:0,
            fy:0,
            radius:1.2,
            stop:0 lightgrey,
            stop:0.8 lightgrey,
            stop:0.94 white,
            stop:1 white
            )
        }}
    QWidget#tab {{ background-color: {0}}}
    QWidget {{ background-color: {0}}}
    QTabBar {{ background-color: {0}}}
    QTabBar::tab {{ min-width: 120px }}
    """.format('lightgrey')

    button_style = """
    QPushButton {{ min-height: {5}}}
    QPushButton {{
        background-color: qradialgradient(
            cx:0.5,
            cy:0.5,
            fx:0.5,
            fy:0.5,
            radius:1,
            stop:0 white,
            stop:1 pink
            );
        border-width: 1px;
        border-style: inset;
        padding: 1px;
        border-radius: 5px
        }}
    QPushButton:pressed {{
        background-color: qradialgradient(
            cx:0.5,
            cy:0.5,
            fx:0.5,
            fy:0.5,
            radius:1,
            stop:0 white,
            stop:1 pink
            );
        border-width: 1px;
        border-style: outset;
        padding: 1px;
        border-radius: 5px
        }}
    QPushButton#button {{ min-width: {0}; max-width: {0} }}
    QPushButton#frame {{ max-width: {1}; min-width: {1} }}
    QPushButton#button_entry {{ max-width: {2}; min-width: {2} }}
    QPushButton#mount {{ max-width: {3}; min-width: {3} }}
    QPushButton#notification {{ max-width: {4}; min-width: {4} }}
    """.format(
        start_button_width,
        frame_label_width,
        frame_button_width,
        mount_button_width,
        notification_button_width,
        widget_height
        )

    label_style = """
    QLabel#picture {{ background-color: transparent }}
    QLabel#status_name {{ max-width: {3}; min-width: {1}; background-color: {0}; min-height: {5}; max-height: {5} }}
    QLabel#status_info {{ max-width: {3}; min-width: {2}; background-color: {0}; min-height: {5}; max-height: {5} }}
    QLabel#status_quota {{ max-width: {3}; min-width: {3}; background-color: {0}; min-height: {5}; max-height: {5} }}
    QLabel#setting {{ max-width: {4}; min-width: {4}; background-color: {0}; min-height: {5}; max-height: {5} }}
    """.format(
        'transparent',
        status_name_width,
        status_info_width,
        status_quota_width,
        setting_widget_width,
        widget_height
        )

    edit_style = """
    QLineEdit {{ max-height: {7}; min-height: {7}; background-color: white }}
    QLineEdit#default_settings {{ min-width: {1}; max-width: 9999; background-color: white }}
    QLineEdit:disabled {{ background-color: {6} }}
    QLineEdit#setting:enabled {{
        max-width: {1}; min-width: {1}; background-color: {0}; min-height: {7}; max-height: {7}
        }}
    QLineEdit#noti_edit:enabled {{
        max-width: {2}; min-width: {2}; background-color: {5}; min-height: {7}; max-height: {7}
        }}
    QLineEdit#frame:enabled {{
        max-width: {3}; min-width: {3}; background-color: {0}; min-height: {7}; max-height: {7}
        }}
    QLineEdit#entry:enabled {{
        max-width: {4}; min-width: {4}; background-color: {0}; min-height: {7}; max-height: {7}
        }}
    """.format(
        'white',
        setting_widget_width,
        notification_edit_width,
        frame_label_width,
        frame_entry_width,
        'rgba(102,205,170,20)',
        'rgba(205,102,170,20)',
        widget_height
        )
        #'rgba(102,205,170,100)',

    check_style = """
    QCheckBox {{ min-height: {1}; max-height: {1}; background-color: white }}
    QCheckBox#noti_check {{ max-width: {0}; min-width: {0}; background-color: white }}
    """.format(notification_check_width, widget_height)

    combo_style = """
    QComboBox {{ min-width: {4}; max-width: {4}; min-height: {3}; max-height: {3}; background-color: white }}
    QComboBox QAbstractItemView {{
        background-color: white; selection-color: black; selection-background-color: lightgray
        }}
    QComboBox#default_settings {{ min-width: {1}; max-width: 9999; background-color: white }}
    QComboBox:disabled {{ background-color: {2} }}
    QComboBox#noti_edit:enabled {{ max-width: {1}; min-width: {1}; background-color: {0} }}
    """.format(
        'rgba(102,205,170,20)',
        notification_edit_width,
        'rgba(195,152,180,20)',
        widget_height,
        setting_widget_width
        )

    style = '\n'.join([style_widgets, button_style, label_style, edit_style, check_style, combo_style])
    return style


def check_instance(value, typ):
    """
    Check typ of value

    Arguments:
    value - Value to check
    typ - Type to check

    Return:
    Bool
    """
    if isinstance(typ, list):
        value_split = value.split(' ')
        for entry, var in zip(typ, value_split):
            try:
                entry(var)
            except ValueError:
                return False
            else:
                pass
    else:
        try:
            typ(value)
        except ValueError:
            return False
        else:
            pass
    return True


def get_style(typ):
    """
    Style colores for the content of widgets.

    Arguments:
    typ - Typ of color

    Return:
    Color string
    """
    if typ == 'error':
        color = 'red'
    elif typ == 'unchanged':
        color = 'black'
    elif typ == 'pass':
        color = 'green'
    elif typ == 'changed':
        color = 'purple'
    elif typ == 'True':
        color = 'blue'
    elif typ == 'False':
        color = 'darkgray'
    else:
        msg = 'Style not known! Go for black!'
        print(msg)
        message(msg)
        color = 'black'

    return 'color: {0}'.format(color)


def import_ctf(ctf_name, directory_name):
    """
    Import ctf information.

    Arguments:
    motion_name - Name of motion program
    directory_name - Name of the directory to search for files

    Return:
    Imported data
    """
    if ctf_name == 'False':
        data, data_orig = None, None
    elif ctf_name == 'Later':
        data, data_orig = None, None
    else:
        data, data_orig = get_function_dict()[ctf_name]['plot_data'](
            ctf_name=ctf_name,
            directory_name=directory_name
            )

    return data, data_orig


def import_motion(motion_name, directory_name):
    """
    Import motion information.

    Arguments:
    motion_name - Name of motion program
    directory_name - Name of the directory to search for files

    Return:
    Imported data
    """
    if motion_name == 'False':
        return None
    elif motion_name == 'Later':
        return None
    else:
        directory_names = glob.glob('{0}/*_with_DW_log'.format(directory_name))
        data = get_function_dict()[motion_name]['plot_data'](
            motion_name=motion_name,
            directory_names=directory_names
            )

    return data



"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import pexpect as pe
try:
    from PyQt4.QtGui import QDialog, QHBoxLayout, QVBoxLayout, QLabel, QPushButton, QLineEdit
    from PyQt4.QtCore import pyqtSlot
except ImportError:
    from PyQt5.QtWidgets import QDialog, QHBoxLayout, QVBoxLayout, QLabel, QPushButton, QLineEdit
    from PyQt5.QtCore import pyqtSlot
from transphire import transphire_utils as tu


class PasswordDialog(QDialog):
    """
    User password dialog

    Inherits:
    QDialog

    Signals:
    None
    """

    def __init__(self, folder, default, login, extension, parent=None):
        """
        Initialise layout of the widget.

        Arguments:
        folder - Folder name to mount.
        default - Default user name
        login - True, if it is possible to test the users credentials on this machine
        extension - True, if a mount extension is required
        parent - Parent widget

        Returns:
        None
        """
        super(PasswordDialog, self).__init__(parent)

        # Global content
        self.user_le = QLineEdit(self)
        self.user_le.setText(default)
        self.password_le = QLineEdit(self)
        self.password_le.setEchoMode(QLineEdit.Password)
        self.folder_le = QLineEdit(self)
        self.folder_le.setText(folder)
        self.folder_le.setReadOnly(True)
        self.add_to_folder = QLineEdit(self)

        self.login = login
        self.extension = extension
        self.add_to_folder.setEnabled(self.extension)

        self.password = None
        self.username = None
        self.folder = None

        # Buttons
        button_accept = QPushButton('OK', self)
        button_reject = QPushButton('Cancel', self)
        layout_button = QHBoxLayout()
        layout_button.setContentsMargins(0, 0, 0, 0)
        layout_button.addWidget(button_accept)
        layout_button.addWidget(button_reject)

        # Layout
        layout = QVBoxLayout(self)
        layout.addWidget(QLabel('Username', self))
        layout.addWidget(self.user_le)
        layout.addWidget(QLabel('Password', self))
        layout.addWidget(self.password_le)
        layout.addWidget(QLabel('Mount device', self))
        layout.addWidget(self.folder_le)
        if self.extension:
            self.add_to_folder.setPlaceholderText('Subfolder of mount device')
            layout.addWidget(QLabel('Mount device extension', self))
            layout.addWidget(self.add_to_folder)
        else:
            self.add_to_folder.hide()
        layout.addLayout(layout_button)

        # Events
        button_accept.clicked.connect(self._my_accept)
        button_reject.clicked.connect(self.reject)

    @pyqtSlot()
    def _my_accept(self):
        """
        Modified accept function.
        """
        self.username = self.user_le.text()
        self.password = self.password_le.text()
        self.folder = os.path.join(
            *self.folder_le.text().split('/'),
            *self.add_to_folder.text().split('/')
            ).rstrip('/')

        if not self.add_to_folder.text().rstrip('/ \t\n').lstrip('/ \t\n'):
            if self.extension:
                tu.message('Folder extension not specified, but required!')
                return None
            else:
                pass
        else:
            pass

        if self.login:
            cmd = 'su - {0} -c \'ls\''.format(self.username)

            process = pe.spawn(cmd)
            idx = process.expect(['Password:', 'No passwd entry for user.*'])
            if idx == 0:
                process.sendline(self.password)
                idx = process.expect([pe.EOF, 'su: Authentication failure'])
                process.interact()
                if idx == 0:
                    self.accept()
                else:
                    print('Wrong password for user: {0}'.format(self.username))
                    self.reject()
            else:
                print('User not known on this machine: {0}'.format(self.username))
                self.reject()

        else:
            self.accept()



"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
try:
    from PyQt4.QtGui import QWidget, QLabel, QHBoxLayout
    from PyQt4.QtCore import pyqtSignal, pyqtSlot
except ImportError:
    from PyQt5.QtWidgets import QWidget, QLabel, QHBoxLayout
    from PyQt5.QtCore import pyqtSignal, pyqtSlot


class StatusWidget(QWidget):
    """
    Widget to show the current status of the processes.

    Inherits:
    QWidget

    Signals:
    sig_change_info_name - Connected, to change the info name information (text|str, color|str)
    sig_change_info_quota - Connected, to change the quota name information (text|str, color|str)
    """
    sig_change_info_name = pyqtSignal(str, str)
    sig_change_info_quota = pyqtSignal(str, str)

    def __init__(self, name, default_name, default_quota, parent=None):
        super(StatusWidget, self).__init__(parent)

        # Global content
        self.info_name = QLabel(default_name, self)
        self.info_name.setObjectName('status_info')
        self.info_name.setStyleSheet('color: purple')
        self.info_quota = QLabel(default_quota, self)
        self.info_quota.setObjectName('status_quota')
        self.info_quota.setStyleSheet('color: purple')

        # Content
        name = QLabel('{0}: '.format(name), self)
        name.setObjectName('status_name')

        # Layout
        layout = QHBoxLayout(self)
        layout.addWidget(name)
        layout.addWidget(self.info_name)
        layout.addWidget(self.info_quota)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.addStretch(1)

        # events
        self.sig_change_info_name.connect(self.change_info_name)
        self.sig_change_info_quota.connect(self.change_info_quota)

    @pyqtSlot(str, str)
    def change_info_name(self, text, color):
        """
        Change the info text and the info text color.

        Arguments:
        text - Text to put
        color - Color to use

        Returns:
        None
        """
        self.info_name.setText(text)
        self.info_name.setStyleSheet('color: {0}'.format(color))

    @pyqtSlot(str, str)
    def change_info_quota(self, text, color):
        """
        Change the info text and the info text color.

        Arguments:
        text - Text to put
        color - Color to use

        Returns:
        None
        """
        self.info_quota.setText(text)
        self.info_quota.setStyleSheet('color: {0}'.format(color))



"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from transphire import transphire_utils as tu


def default_cter_v1_0():
    """
    Content of CTER SPHIRE version 1.0.

    Arguments:
    None

    Return:
    Content items as list
    """
    items = [
        ['--wn', '512', float, '', 'PLAIN'],
        ['--apix', '-1', float, '', 'PLAIN'],
        ['--Cs', '2.0', float, '', 'PLAIN'],
        ['--voltage', '300', float, '', 'PLAIN'],
        ['--ac', '10', float, '', 'PLAIN'],
        ['--f_start', '-1', float, '', 'PLAIN'],
        ['--f_stop', '-1', float, '', 'PLAIN'],
        ['--kboot', '16', float, '', 'PLAIN'],
        ['--overlap_x', '50', float, '', 'PLAIN'],
        ['--overlap_y', '50', float, '', 'PLAIN'],
        ['--edge_x', '0', float, '', 'PLAIN'],
        ['--pap', ['False', 'True'], bool, '', 'COMBO'],
        ['Phase plate', ['False', 'True'], bool, '', 'COMBO'],
        ['--defocus_min', '0.3', float, 'Phase plate:True', 'PLAIN'],
        ['--defocus_max', '9.0', float, 'Phase plate:True', 'PLAIN'],
        ['--defocus_step', '0.1', float, 'Phase plate:True', 'PLAIN'],
        ['--phase_min', '5', float, 'Phase plate:True', 'PLAIN'],
        ['--phase_max', '175', float, 'Phase plate:True', 'PLAIN'],
        ['--phase_step', '5', float, 'Phase plate:True', 'PLAIN'],
        ]
    return items


def default_gctf_v1_06():
    """
    Content of GCtf version 1.06.

    Arguments:
    None

    Return:
    Content items as list
    """
    items = [
        ['--apix', '1.34', float, '', 'PLAIN'],
        ['--dstep', '14.0', float, '', 'PLAIN'],
        ['--kV', '300', float, '', 'PLAIN'],
        ['--cs', '2.7', float, '', 'PLAIN'],
        ['--ac', '0.1', float, '', 'PLAIN'],
        ['Phase plate', ['False', 'True'], bool, '', 'COMBO'],
        ['--phase_shift_L', '0.0', float, 'Phase plate:True', 'PLAIN'],
        ['--phase_shift_H', '180.0', float, 'Phase plate:True', 'PLAIN'],
        ['--phase_shift_S', '5.0', float, 'Phase plate:True', 'PLAIN'],
        ['--phase_shift_T', '1', int, 'Phase plate:True', 'PLAIN'],
        ['--bfac', '150', float, '', 'PLAIN'],
        ['--defL', '5000', float, '', 'PLAIN'],
        ['--defH', '90000', float, '', 'PLAIN'],
        ['--defS', '500', float, '', 'PLAIN'],
        ['--astm', '1000', float, '', 'PLAIN'],
        ['--resL', '50', float, '', 'PLAIN'],
        ['--resH', '4', float, '', 'PLAIN'],
        ['--boxsize', '1024', int, '', 'PLAIN'],
        ['--overlap', '0.5', float, '', 'PLAIN'],
        ['--convsize', '85', float, '', 'PLAIN'],
        ['--do_EPA', '0', int, '', 'PLAIN'],
        ['--do_Hres_ref', '0', int, '', 'PLAIN'],
        ['--Href_resL', '15.0', float, '', 'PLAIN'],
        ['--Href_resH', '4.0', float, '', 'PLAIN'],
        ['--Href_bfac', '50', float, '', 'PLAIN'],
        ['--refine_after_EPA', '0', int, '', 'PLAIN'],
        ['--estimate_B', '1', int, '', 'PLAIN'],
        ['--B_resL', '15', float, '', 'PLAIN'],
        ['--B_resH', '6', float, '', 'PLAIN'],
        ['Use movies', ['False', 'True'], bool, '', 'COMBO'],
        ['--do_mdef_refine', '0', int, 'Use movies:True', 'PLAIN'],
        ['--mdef_ave_type', '0', int, 'Use movies:True', 'PLAIN'],
        ['--mdef_aveN', '7', int, 'Use movies:True', 'PLAIN'],
        ['--mdef_fit', '0', int, 'Use movies:True', 'PLAIN'],
        ['--do_local_refine', '0', int, '', 'PLAIN'],
        ['--local_radius', '1024', float, '', 'PLAIN'],
        ['--local_boxsize', '512', float, '', 'PLAIN'],
        ['--local_overlap', '0.5', float, '', 'PLAIN'],
        ['--local_avetype', '0', float, '', 'PLAIN'],
        ['--do_phase_flip', '0', int, '', 'PLAIN'],
        ['--do_validation', '0', int, '', 'PLAIN'],
        ['--ctfout_resL', '100.0', float, '', 'PLAIN'],
        ['--ctfout_resH', '2.8', float, '', 'PLAIN'],
        ['--ctfout_bfac', '50', float, '', 'PLAIN'],
        ['--gid', '0', [int]*9, '', 'PLAIN']
        ]
    return items


def default_gctf_v1_18():
    """
    Content of GCtf version 1.18.

    Arguments:
    None

    Return:
    Content items as list
    """
    items = [
        ['--apix', '1.34', float, '', 'PLAIN'],
        ['--dstep', '14.0', float, '', 'PLAIN'],
        ['--kV', '300', float, '', 'PLAIN'],
        ['--cs', '2.7', float, '', 'PLAIN'],
        ['--ac', '0.1', float, '', 'PLAIN'],
        ['Phase plate', ['False', 'True'], bool, '', 'COMBO'],
        ['--phase_shift_L', '0.0', float, 'Phase plate:True', 'PLAIN'],
        ['--phase_shift_H', '180.0', float, 'Phase plate:True', 'PLAIN'],
        ['--phase_shift_S', '0.0', float, 'Phase plate:True', 'PLAIN'],
        ['--phase_shift_T', '1', int, 'Phase plate:True', 'PLAIN'],
        ['--only_search_ps', '0', int, 'Phase plate:True', 'PLAIN'],
        ['--cosearch_refine_ps', '0', int, 'Phase plate:True', 'PLAIN'],
        ['--bfac', '150', float, '', 'PLAIN'],
        ['--defL', '5000', float, '', 'PLAIN'],
        ['--defH', '90000', float, '', 'PLAIN'],
        ['--defS', '500', float, '', 'PLAIN'],
        ['--astm', '1000', float, '', 'PLAIN'],
        ['--resL', '50', float, '', 'PLAIN'],
        ['--resH', '4', float, '', 'PLAIN'],
        ['--boxsize', '1024', int, '', 'PLAIN'],
        ['--overlap', '0.5', float, '', 'PLAIN'],
        ['--convsize', '85', float, '', 'PLAIN'],
        ['--refine_2d_T', '1', int, '', 'PLAIN'],
        ['--smooth_resL', '0', float, '', 'PLAIN'],
        ['--do_basic_rotave', '0', int, '', 'PLAIN'],
        ['--do_EPA', '0', int, '', 'PLAIN'],
        ['--refine_after_EPA', '0', int, '', 'PLAIN'],
        ['--do_Hres_ref', '0', int, '', 'PLAIN'],
        ['--Href_resL', '15.0', float, '', 'PLAIN'],
        ['--Href_resH', '4.0', float, '', 'PLAIN'],
        ['--Href_bfac', '50', float, '', 'PLAIN'],
        ['--Href_PS_err', '2', float, '', 'PLAIN'],
        ['--Href_U_err', '100', float, '', 'PLAIN'],
        ['--Href_V_err', '100', float, '', 'PLAIN'],
        ['--Href_A_err', '5', float, '', 'PLAIN'],
        ['--estimate_B', '1', int, '', 'PLAIN'],
        ['--B_resL', '15', float, '', 'PLAIN'],
        ['--B_resH', '6', float, '', 'PLAIN'],
        ['Use movies', ['False', 'True'], bool, '', 'COMBO'],
        ['--do_mdef_refine', '0', int, 'Use movies:True', 'PLAIN'],
        ['--mdef_ave_type', '0', int, 'Use movies:True', 'PLAIN'],
        ['--mdef_aveN', '7', int, 'Use movies:True', 'PLAIN'],
        ['--mdef_fit', '0', int, 'Use movies:True', 'PLAIN'],
        ['--do_local_refine', '0', int, '', 'PLAIN'],
        ['--local_radius', '1024', float, '', 'PLAIN'],
        ['--local_boxsize', '512', float, '', 'PLAIN'],
        ['--local_overlap', '0.5', float, '', 'PLAIN'],
        ['--local_avetype', '0', float, '', 'PLAIN'],
        ['--do_phase_flip', '0', int, '', 'PLAIN'],
        ['--do_validation', '0', int, '', 'PLAIN'],
        ['--ctfout_resL', '100.0', float, '', 'PLAIN'],
        ['--ctfout_resH', '2.78', float, '', 'PLAIN'],
        ['--ctfout_bfac', '50', float, '', 'PLAIN'],
        ['--gid', '0', [int]*9, '', 'PLAIN']
        ]
    return items


def default_path():
    """
    Content of Path tab.

    Arguments:
    None

    Return:
    Content items as list
    """
    items = [['IMOD header', '', str, '', 'FILE/CHOICE']]
    items.append(['IMOD newstack', '', str, '', 'FILE/CHOICE'])
    items.append(['IMOD mrc2tif', '', str, '', 'FILE/CHOICE'])
    items.append(['IMOD dm2mrc', '', str, '', 'FILE/CHOICE'])
    items.append(['SumMovie v1.0.2', '', str, '', 'FILE/CHOICE'])
    function_dict = tu.get_function_dict()
    for key in sorted(function_dict.keys()):
        if function_dict[key]['executable']:
            items.append([key, '', str, '', 'FILE/CHOICE'])
        else:
            pass

    return items


def default_font():
    """
    Content of Font tab.

    Arguments:
    None

    Return:
    Content items as list
    """
    items = [
        ['Font', '15', float, '', 'PLAIN'],
        ['Width adjustment', '1', float, '', 'PLAIN'],
        ['Height adjustment', '1', float, '', 'PLAIN'],
        ['Start button', '10', float, '', 'PLAIN'],
        ['Notification edit', '16', float, '', 'PLAIN'],
        ['Notification check', '8', float, '', 'PLAIN'],
        ['Notification button', '25', float, '', 'PLAIN'],
        ['Mount button', '16', float, '', 'PLAIN'],
        ['Frame entry', '5', float, '', 'PLAIN'],
        ['Frame button', '8', float, '', 'PLAIN'],
        ['Frame label', '8', float, '', 'PLAIN'],
        ['Setting widget', '22', float, '', 'PLAIN'],
        ['Status name', '12', float, '', 'PLAIN'],
        ['Status info', '12', float, '', 'PLAIN'],
        ['Status quota', '12', float, '', 'PLAIN'],
        ['Widget height', '1.5', float, '', 'PLAIN']
        ]
    return items


def default_others():
    """
    Content of Status tab.

    Arguments:
    None

    Return:
    Content items as list
    """
    items = [
        ['Image', '', str, '', 'FILE'],
        ['Mount/umount needs sudo password?', ['True', 'False'], bool, '', 'COMBO'],
        ]
    return items


def default_notification():
    """
    Content of notification tab.

    Arguments:
    None

    Return:
    Content items as list
    """
    items = [
        ['Bot token', '', str, '', 'PLAIN'],
        [
            'Default names telegram',
            'name1:telegram_name1;name2:telegram_name2',
            str,
            '',
            'PLAIN'
            ],
        ['SMTP server', '', str, '', 'PLAIN'],
        ['E-Mail adress', '', str, '', 'PLAIN'],
        [
            'Default names email',
            'name1:telegram_name1;name2:telegram_name2',
            str,
            '',
            'PLAIN'
            ],
        ['Number of users', '3', int, '', 'PLAIN']
        ]
    return items


def default_pipeline():
    """
    Content of pipeline tab.

    Arguments:
    None

    Return:
    Content items as list
    """
    items = [
        [
            'Meta',
            '1',
            int,
            'Meta;' +
            'Session to work:Copy to work:Copy_work,' +
            'Session to backup:Copy to backup:Copy_backup,' +
            'Session to HDD:Copy to HDD:Copy_hdd',
            'PLAIN'
            ],
        ['Find', '1', int, 'Find;Copy', 'PLAIN'],
        [
            'Copy',
            '1',
            int,
            'Find;' +
            'Motion:Motion,' +
            'Meta to work:Copy to work:Copy_work,' +
            'Meta to backup:Copy to backup:Copy_backup,' +
            'Meta to HDD:Copy to HDD:Copy_hdd,' +
            '!Motion:CTF_frames:CTF,' +
            '!Motion:!CTF_frames:Compress data:Compress,' +
            '!Motion:!CTF_frames:!Compress data:Frames to work:Copy to work:Copy_work,' +
            '!Motion:!CTF_frames:!Compress data:Frames to HDD:Copy to HDD:Copy_hdd,' +
            '!Motion:!CTF_frames:!Compress data:Frames to backup:Copy to backup:Copy_backup',
            'PLAIN'
            ],
        [
            'Motion',
            '2',
            int,
            'Motion;' +
            'Sum to work:Copy to work:Copy_work,' +
            'Sum to HDD:Copy to HDD:Copy_hdd,' +
            'Sum to backup:Copy to backup:Copy_backup,' +
            'CTF_frames:CTF,' +
            'CTF_sum:CTF,' +
            '!CTF_frames:Compress data:Compress,' +
            '!CTF_frames:!Compress data:Frames to work:Copy to work:Copy_work,' +
            '!CTF_frames:!Compress data:Frames to HDD:Copy to HDD:Copy_hdd,' +
            '!CTF_frames:!Compress data:Frames to backup:Copy to backup:Copy_backup',
            'PLAIN'
            ],
        [
            'CTF',
            '2',
            int,
            'CTF;' +
            'CTF to work:Copy to work:Copy_work,' +
            'CTF to HDD:Copy to HDD:Copy_hdd,' +
            'CTF to backup:Copy to backup:Copy_backup,' +
            'CTF_frames:Compress data:Compress,' +
            'CTF_frames:!Compress data:Frames to work:Copy to work:Copy_work,' +
            'CTF_frames:!Compress data:Frames to HDD:Copy to HDD:Copy_hdd,' +
            'CTF_frames:!Compress data:Frames to backup:Copy to backup:Copy_backup',
            'PLAIN'
            ],
        [
            'Compress',
            '2',
            int,
            'Compress;'
            'Frames to backup:Copy to backup:Copy_backup,'
            'Frames to HDD:Copy to HDD:Copy_hdd,'
            'Frames to work:Copy to work:Copy_work',
            'PLAIN'
            ],
        ['Copy_work', '1', int, 'Copy_work;', 'PLAIN'],
        ['Copy_hdd', '1', int, 'Copy_hdd;', 'PLAIN'],
        ['Copy_backup', '1', int, 'Copy_backup;', 'PLAIN'],
        ]
    return items


def default_ctffind_4_v4_1_10():
    """
    Content of CTFFind version 4.1.10.

    Arguments:
    None

    Return:
    Content items as list
    """
    return default_ctffind_4_v4_1_8()


def default_ctffind_4_v4_1_8():
    """
    Content of CTFFind version 4.1.8.

    Arguments:
    None

    Return:
    Content items as list
    """
    items = [
        ['Pixel size', '1.0', float, '', 'PLAIN'],
        ['Acceleration voltage', '300.0', float, '', 'PLAIN'],
        ['Spherical aberration', '2.7', float, '', 'PLAIN'],
        ['Amplitude contrast', '0.07', float, '', 'PLAIN'],
        ['Amplitude spectrum', '512', float, '', 'PLAIN'],
        ['Min resolution(A)', '30', float, '', 'PLAIN'],
        ['Max resolution(A)', '5', float, '', 'PLAIN'],
        ['Min defocus(A)', '5000', float, '', 'PLAIN'],
        ['Max defocus(A)', '50000', float, '', 'PLAIN'],
        ['Step defocus(A)', '500', float, '', 'PLAIN'],
        ['High accuracy', ['True', 'False'], bool, '', 'COMBO'],
        ['Know astigmatism', ['True', 'False'], bool, '', 'COMBO'],
        ['Restrain astigmatism', ['True', 'False'], bool, 'Know astigmatism:False', 'COMBO'],
        ['Expected astigmatism', '200', float, 'Restrain astigmatism:True', 'PLAIN'],
        ['Astigmatism', '0', float, 'Know astigmatism:True', 'PLAIN'],
        ['Astigmatism angle', '0', float, 'Know astigmatism:True', 'PLAIN'],
        ['Phase shift', ['False', 'True'], bool, '', 'COMBO'],
        ['Min phase(rad)', '0', float, 'Phase shift:True', 'PLAIN'],
        ['Max phase(rad)', '3.15', float, 'Phase shift:True', 'PLAIN'],
        ['Step phase(rad)', '0.5', float, 'Phase shift:True', 'PLAIN'],
        ['Resample micrographs', ['True', 'False'], bool, '', 'COMBO'],
        ['Use movies', ['False', 'True'], bool, '', 'COMBO'],
        ['Combine frames', '1', int, 'Use movies:True', 'PLAIN'],
        ['Movie is gain-corrected?', ['True', 'False'], bool, 'Use movies:True', 'COMBO'],
        ['Gain file', '', str, 'Movie is gain-corrected?:False', 'FILE'],
        ['Correct mag. distort.', ['False', 'True'], bool, 'Use movies:True', 'COMBO'],
        ['Mag. dist. angle', '0.0', float, 'Correct mag. distort.:True', 'PLAIN'],
        ['Mag. dist. major scale', '1.0', float, 'Correct mag. distort.:True', 'PLAIN'],
        ['Mag. dist. minor scale', '1.0', float, 'Correct mag. distort.:True', 'PLAIN'],
        ]
    return items


def default_mount(hdd=None):
    """
    Content of Mount tab.

    Arguments:
    hdd - Content is related to HDD

    Return:
    Content items as list
    """
    if hdd is not None:
        name = 'HDD'
    else:
        name = ''
    items = [
        ['Mount name', name, str, '', 'PLAIN'],
        ['Protocol', ['smbfs', 'cifs', 'nfs'], str, '', 'COMBO'],
        ['Protocol version', '2.0', float, '', 'PLAIN'],
        ['sec', ['ntlmssp', 'krb5', 'krb5i', 'ntlm', 'ntlmi', 'ntlmv2', 'ntlmv2i', 'ntlmsspi', 'none'], str, '', 'COMBO'],
        ['gid', '', str, '', 'PLAIN'],
        ['Domain', '', str, '', 'PLAIN'],
        ['IP', '', str, '', 'PLAIN'],
        ['Folder', '', str, '', 'PLAIN'],
        ['Need folder extension?', ['False', 'True'], bool, '', 'COMBO'],
        ['Default user', '', str, '', 'PLAIN'],
        ['Is df giving the right quota?', ['True', 'False'], bool, '', 'COMBO'],
        ['Target UID exists here and on target?', ['True', 'False'], bool, '', 'COMBO'],
        ['Need sudo for copy?', ['False', 'True'], bool, '', 'COMBO'],
        ['SSH address', '', str, '', 'PLAIN'],
        ['Quota command', '', str, '', 'PLAIN'],
        ['Quota / TB', '', float, '', 'PLAIN'],
        ['Typ', ['Copy', 'Copy_work', 'Copy_backup'], str, '', 'COMBO'],
        ]
    return items


def default_motion_cor_2_v1_0_0():
    """
    Content of MotionCor2 version 1.0.0.

    Arguments:
    None

    Return:
    Content items as list
    """
    items = [
        ['-FmDose', '0', float, '', 'PLAIN'],
        ['-PixSize', '0', float, '', 'PLAIN'],
        ['-kV', '300', float, '', 'PLAIN'],
        ['-MaskCent', '0 0', [float, float], '', 'PLAIN'],
        ['-MaskSize', '1 1', [float, float], '', 'PLAIN'],
        ['-Patch', '0 0', [int, int], '', 'PLAIN'],
        ['-Iter', '7', int, '', 'PLAIN'],
        ['-Tol', '0.5', float, '', 'PLAIN'],
        ['-Bft', '100', float, '', 'PLAIN'],
        ['-PhaseOnly', '0', int, '', 'PLAIN'],
        ['-StackZ', '0', int, '', 'PLAIN'],
        ['-FtBin', '1', float, '', 'PLAIN'],
        ['-InitDose', '0', float, '', 'PLAIN'],
        ['-Throw', '0', int, '', 'PLAIN'],
        ['-Trunc', '0', int, '', 'PLAIN'],
        ['-Group', '1', int, '', 'PLAIN'],
        ['-FmRef', '-1', int, '', 'PLAIN'],
        ['-DefectFile', '', str, '', 'FILE'],
        ['-Gain', '', str, '', 'FILE'],
        ['-RotGain', '0', int, '', 'PLAIN'],
        ['-FlipGain', '0', int, '', 'PLAIN'],
        ['-Tilt', '0 0', [float, float], '', 'PLAIN'],
        ['-Mag', '1 1 0', [float, float, float], '', 'PLAIN'],
        ['-InFmMotion', '0', int, '', 'PLAIN'],
        ['-Crop', '0 0', [int, int], '', 'PLAIN'],
        ['-Gpu', '0', [int]*9, '', 'PLAIN'],
        ]
    return items


def default_motion_cor_2_v1_0_5():
    """
    Content of MotionCor2 version 1.0.5.

    Arguments:
    None

    Return:
    Content items as list
    """
    items = [
        ['-FmDose', '0', float, '', 'PLAIN'],
        ['-PixSize', '0', float, '', 'PLAIN'],
        ['-kV', '300', float, '', 'PLAIN'],
        ['-MaskCent', '0 0', [float, float], '', 'PLAIN'],
        ['-MaskSize', '1 1', [float, float], '', 'PLAIN'],
        ['-Patch', '0 0 0', [int, int, int], '', 'PLAIN'],
        ['-Iter', '7', int, '', 'PLAIN'],
        ['-Tol', '0.5', float, '', 'PLAIN'],
        ['-Bft', '100', float, '', 'PLAIN'],
        ['-PhaseOnly', '0', int, '', 'PLAIN'],
        ['-StackZ', '0', int, '', 'PLAIN'],
        ['-FtBin', '1', float, '', 'PLAIN'],
        ['-InitDose', '0', float, '', 'PLAIN'],
        ['-Throw', '0', int, '', 'PLAIN'],
        ['-Trunc', '0', int, '', 'PLAIN'],
        ['-Group', '1', int, '', 'PLAIN'],
        ['-FmRef', '-1', int, '', 'PLAIN'],
        ['-DefectFile', '', str, '', 'FILE'],
        ['-Dark', '', str, '', 'FILE'],
        ['-Gain', '', str, '', 'FILE'],
        ['-RotGain', '0', int, '', 'PLAIN'],
        ['-FlipGain', '0', int, '', 'PLAIN'],
        ['-Tilt', '0 0', [float, float], '', 'PLAIN'],
        ['-Mag', '1 1 0', [float, float, float], '', 'PLAIN'],
        ['-InFmMotion', '0', int, '', 'PLAIN'],
        ['-Crop', '0 0', [int, int], '', 'PLAIN'],
        ['-Gpu', '0', [int]*9, '', 'PLAIN'],
        ]
    return items


def default_motion_cor_2_v1_1_0():
    """
    Content of MotionCor2 version 1.1.0.

    Arguments:
    None

    Return:
    Content items as list
    """
    items = [
        ['-FmDose', '0', float, '', 'PLAIN'],
        ['-PixSize', '0', float, '', 'PLAIN'],
        ['-kV', '300', float, '', 'PLAIN'],
        ['-MaskCent', '0 0', [float, float], '', 'PLAIN'],
        ['-MaskSize', '1 1', [float, float], '', 'PLAIN'],
        ['-Patch', '0 0 0', [int, int, int], '', 'PLAIN'],
        ['-Iter', '7', int, '', 'PLAIN'],
        ['-Tol', '0.5', float, '', 'PLAIN'],
        ['-Bft', '500 150', [float, float], '', 'PLAIN'],
        ['-PhaseOnly', '0', int, '', 'PLAIN'],
        ['-StackZ', '0', int, '', 'PLAIN'],
        ['-FtBin', '1', float, '', 'PLAIN'],
        ['-InitDose', '0', float, '', 'PLAIN'],
        ['-Throw', '0', int, '', 'PLAIN'],
        ['-Trunc', '0', int, '', 'PLAIN'],
        ['-Group', '1', int, '', 'PLAIN'],
        ['-FmRef', '-1', int, '', 'PLAIN'],
        ['-DefectFile', '', str, '', 'FILE'],
        ['-Dark', '', str, '', 'FILE'],
        ['-Gain', '', str, '', 'FILE'],
        ['-RotGain', '0', int, '', 'PLAIN'],
        ['-FlipGain', '0', int, '', 'PLAIN'],
        ['-Tilt', '0 0', [float, float], '', 'PLAIN'],
        ['-Mag', '1 1 0', [float, float, float], '', 'PLAIN'],
        ['-InFmMotion', '0', int, '', 'PLAIN'],
        ['-Crop', '0 0', [int, int], '', 'PLAIN'],
        ['-Gpu', '0', [int]*9, '', 'PLAIN'],
        ['-GpuMemUsage', '0.5', float, '', 'PLAIN'],
        ]
    return items


def default_general():
    """
    Content of General tab.

    Arguments:
    None

    Return:
    Content items as list
    """
    items = [
        ['Software', ['EPU 1.9', 'EPU 1.8'], str, '', 'COMBO'],
        ['Type', ['Stack', 'Frames'], str, '', 'COMBO'],
        ['Camera', ['K2', 'Falcon3', 'Falcon2'], str, '', 'COMBO'],
        ['Search path frames', '', str, '', 'DIR'],
        ['Search path meta', '', str, '', 'DIR'],
        ['Input extension', ['mrc', 'dm4', 'tif', 'tiff'], str, '', 'COMBO'],
        ['Output extension', ['mrc', 'tif', 'tiff'], str, '', 'COMBO'],
        ['Project name', '', str, '', 'PLAIN'],
        ['Number of frames', '0', int, '', 'PLAIN'],
        ['Rename micrographs', ['True', 'False'], bool, '', 'COMBO'],
        ['Rename prefix', '', str, 'Rename micrographs:True', 'PLAIN'],
        ['Rename suffix', '', str, 'Rename micrographs:True', 'PLAIN'],
        ['Start number', '0', int, 'Rename micrographs:True', 'PLAIN'],
        ['Estimated mic number', '5000', int, 'Rename micrographs:True', 'PLAIN'],
        ['Project directory', '', str, '', 'DIR'],
        ['Project quota warning (%)', '80', float, '', 'PLAIN'],
        ['Project quota stop (%)', '90', float, '', 'PLAIN'],
        ['Scratch directory', '', str, '', 'DIR'],
        ['Scratch quota warning (%)', '80', float, '', 'PLAIN'],
        ['Scratch quota stop (%)', '90', float, '', 'PLAIN'],
        ['Time until notification', '25', float, '', 'PLAIN'],
        ['Phase shift warning (deg)', '110', float, '', 'PLAIN'],
        ]
    return items


def default_copy(settings_folder):
    """
    Content of Copy tab.

    Arguments:
    None

    Return:
    Content items as list
    """
    extend_list = ['Later', 'False']
    mount_dict = tu.get_key_names(
        settings_folder=settings_folder,
        name='Mount'
        )
    programs_motion, programs_ctf = tu.reduce_programs()

    copy_work = sorted(mount_dict['Copy_work'])
    copy_backup = sorted(mount_dict['Copy_backup'])
    copy_hdd = sorted(mount_dict['Copy_hdd'])

    copy_work.extend(extend_list)
    copy_backup.extend(extend_list)
    copy_hdd.extend(extend_list)
    programs_motion.extend(extend_list)
    programs_ctf.extend(extend_list)

    items = [
        ['Session to work', ['False', 'True'], bool, '', 'COMBO'],
        ['Session to HDD', ['False', 'True'], bool, '', 'COMBO'],
        ['Session to backup', ['False', 'True'], bool, '', 'COMBO'],
        ['Frames to work', ['False', 'True'], bool, '', 'COMBO'],
        ['Frames to HDD', ['False', 'True'], bool, '', 'COMBO'],
        ['Frames to backup', ['False', 'True'], bool, '', 'COMBO'],
        ['Meta to work', ['False', 'True'], bool, '', 'COMBO'],
        ['Meta to HDD', ['False', 'True'], bool, '', 'COMBO'],
        ['Meta to backup', ['False', 'True'], bool, '', 'COMBO'],
        ['Sum to work', ['False', 'True'], bool, '', 'COMBO'],
        ['Sum to HDD', ['False', 'True'], bool, '', 'COMBO'],
        ['Sum to backup', ['False', 'True'], bool, '', 'COMBO'],
        ['CTF to work', ['False', 'True'], bool, '', 'COMBO'],
        ['CTF to HDD', ['False', 'True'], bool, '', 'COMBO'],
        ['CTF to backup', ['False', 'True'], bool, '', 'COMBO'],
        ['Copy to work', copy_work, bool, '', 'COMBO'],
        ['Copy to backup', copy_backup, bool, '', 'COMBO'],
        ['Copy to HDD', copy_hdd, bool, '', 'COMBO'],
        ['Motion', programs_motion, bool, '', 'COMBO'],
        ['CTF', programs_ctf, bool, '', 'COMBO'],
        ['Compress data', ['True', 'Later', 'False'], bool, '', 'COMBO'],
        ['Delete stack after compression?', ['True', 'False'], bool, '', 'COMBO'],
        ]
    return items



"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import json
try:
    from PyQt4.QtGui import QDialog, QVBoxLayout, QPushButton, QTabWidget
except ImportError:
    from PyQt5.QtWidgets import QDialog, QVBoxLayout, QPushButton, QTabWidget

from transphire.loadcontentcontainer import LoadContentContainer
from transphire.separator import Separator
from transphire import transphire_utils as tu


class DefaultSettings(QDialog):
    """
    DefaultSettings dialog.
    Dialog used to enter default values.

    Inherits from:
    QWidget
    """
    def __init__(self, apply, parent=None):
        """
        Setup the layout for the widget.

        Arguments:
        apply - If True, the changes will be applied after saving.
        parent - Parent widget (default None)

        Return:
        None
        """

        super(DefaultSettings, self).__init__(parent)

        # Add new tab widget for the settings to layout
        layout = QVBoxLayout(self)
        self.tab_widget = QTabWidget(self)
        layout.addWidget(self.tab_widget)
        self.tab_widget.setObjectName('tab')

        self.default_tabs = {
            'External software': [
                [
                    'CTFFIND4',
                    'Gctf',
                    'CTER',
                    'MotionCor2'
                    ],
                QTabWidget(self)
                ],
            'TranSPHIRE settings': [
                [
                    'Mount',
                    'Pipeline',
                    'General',
                    'Notification',
                    'Others',
                    'Font',
                    'Copy',
                    'Path'
                    ],
                QTabWidget(self)
                ]
            }
        for tab_name in self.default_tabs:
            self.default_tabs[tab_name][1].setObjectName('tab')
            self.tab_widget.addTab(self.default_tabs[tab_name][1], tab_name)

        # Variables
        self.apply = False
        self.overwrite = True

        # Add buttons
        done_button = QPushButton('Done', self)
        done_button.clicked.connect(lambda: self.check_modified_widgets(done=True))
        layout.addWidget(Separator(typ='horizontal', color='blue'))
        layout.addWidget(done_button)
        if apply:
            layout.addWidget(Separator(typ='horizontal', color='blue'))
            apply_button = QPushButton('Apply', self)
            apply_button.clicked.connect(lambda: self.check_modified_widgets(done=False))
            layout.addWidget(apply_button)

    def check_modified_widgets(self, done):
        """
        Check, if a widget is modified before saving.

        Arguments:
        done - If True, close after input, else apply settings.

        Return:
        True if no modification, else False
        """
        text_modified = False
        wrongly_mod_list = []
        for idx in range(self.tab_widget.count()):
            widget = self.tab_widget.widget(idx)
            for idx_subtab in range(widget.count()):
                widget_subtab = widget.widget(idx_subtab)
                for entry in widget_subtab.content:
                    for info in entry.content:
                        if tu.get_style(typ='changed') == info['widget'].styleSheet() or \
                                tu.get_style(typ='error') == info['widget'].styleSheet():
                            text_modified = True
                            wrongly_mod_list.append(info)
                        else:
                            continue

        if text_modified:
            result = tu.question(
                head='Unsaved changes',
                text='You do have unsaved changes!\nDo you really want to continue?.',
                parent=self
                )
        else:
            result = False

        # Result is True if answer is No
        if result:
            return False
        elif done:
            for info in wrongly_mod_list:
                info['widget'].setText(info['settings']['values'])
            self.accept()
        else:
            for info in wrongly_mod_list:
                info['widget'].setText(info['settings']['values'])
            self.accept_apply()

        return True

    def closeEvent(self, event):
        """
        Handle the close event.

        Arguments:
        event - Close event

        Return:
        None
        """
        if self.check_modified_widgets(done=True):
            pass
        else:
            event.ignore()

    def accept_apply(self):
        """
        Set the apply settings variable to True before saving.

        Argumens:
        None

        Return:
        None
        """
        self.apply = True
        self.accept()

    def add_tab(self, widget, name):
        """
        Add a new widget to the tab widget

        Arguments:
        widget - Widget to add as new tab
        Name - Name of the new tab

        Return:
        None
        """
        is_inside = False
        for key in self.default_tabs:
            compare = self.default_tabs[key][0]
            tab_widget = self.default_tabs[key][1]
            for tab_name in compare:
                if name.startswith(tab_name):
                    is_inside = True
                    tab_widget.addTab(widget, name)
                else:
                    pass
        if not is_inside:
            raise OSError('Name {0} not known! Please fix before continuation'.format(name))
        else:
            pass

    def get_apply(self):
        """
        Getter for the self.apply variable.

        Arguments:
        None

        Return:
        Content of self.apply
        """
        return self.apply

    @staticmethod
    def get_content_default(
            edit_settings,
            apply,
            settings_folder
            ):
        """
        Staticmethod to open the default content dialog.

        Arguments:
        edit_settings - If True, open default settings dialog, else just return content
        apply - Apply settings after closing the default settings dialog
        settings_folder - Folder to store the default settings

        Return:
        Content for the widgets, Content of the apply variable
        """
        # Initialise default settings
        setting_names = sorted(tu.get_function_dict().keys())
        default_widget = DefaultSettings(apply=apply)

        # Initialise a new LoadContentContainer and add it as a new tab
        # Load default settings from settings file into LoadContentContainer
        content_temp = {}
        for name in setting_names:
            default_file = '{0}/content_{1}.txt'.format(settings_folder, name.replace(' ', '_'))
            content_temp[name] = LoadContentContainer(
                typ=name,
                file_name=default_file,
                settings_folder=settings_folder
                )
            default_widget.add_tab(widget=content_temp[name], name=name)
            try:
                with open(default_file, 'r') as file_r:
                    settings = json.load(file_r)
            except FileNotFoundError:
                pass
            else:
                content_temp[name].set_settings(settings)

        # If edit settings, open default settings dialog
        if edit_settings:
            result = default_widget.exec_()
            if result:
                apply = default_widget.get_apply()
            else:
                apply = None
        else:
            apply = None

        # Refresh content of LoadContentContainer by the provided default settings
        content = {}
        for name in setting_names:
            default_file = '{0}/content_{1}.txt'.format(settings_folder, name.replace(' ', '_'))
            content[name] = content_temp[name].get_settings()
            if name == 'Mount':
                continue
            elif not os.path.exists(default_file):
                print('INFORMATION: {0} default settings not modified!'.format(name))
            else:
                with open(default_file, 'r') as file_r:
                    data = json.load(file_r)
                    for entry in data:
                        for dictionary in entry:
                            for name_content in content[name]:
                                for default_value in name_content:
                                    if default_value.keys() == dictionary.keys():
                                        for key in default_value:
                                            default = dictionary[key][0]
                                            default_value[key][0] = default
                                        break
                                    else:
                                        pass
        return content, apply



#! /usr/bin/env python
"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import sys
import os
import json
import argparse
try:
    from PyQt4.QtGui import QApplication
except ImportError:
    from PyQt5.QtWidgets import QApplication

import transphire
from transphire import transphire_utils as tu
from transphire.mainwindow import MainWindow
from transphire.loadwindow import DefaultSettings


def main(font, root_directory, settings_directory, mount_directory, adjust_width, adjust_height, edit_settings):
    """
    Run the GUI.

    Arguments:
    font - Default value for the font size
    adjust_width - Value to adjust the width of buttons
    adjust_height - Value to adjust the height of buttons
    edit_settings - If True, open the default settings dialog

    Return:
    None
    """
    # Start the GUI from the users home directory.
    os.chdir(root_directory)

    app = QApplication([])
    app.setStyle('cleanlooks')

    # Create default folders
    tu.mkdir_p(settings_directory)
    tu.mkdir_p(mount_directory)

    # Load default font settings for the default settings dialog
    if font is None:
        try:
            with open('{0}/content_Font.txt'.format(settings_directory), 'r') as file_r:
                settings = json.load(file_r)
            app.setStyleSheet(
                tu.look_and_feel_small(
                    app=app,
                    font=settings[0][0]['Font'][0]
                    )
                )
        except FileNotFoundError:
            pass
    else:
        app.setStyleSheet(tu.look_and_feel_small(app=app, font=font))

    # Get default settings 0: content 
    IDX_CONTENT = 0
    content = DefaultSettings.get_content_default(
        edit_settings=edit_settings,
        apply=False,
        settings_folder=settings_directory
        )[IDX_CONTENT]

    # Apply font settings to application
    app.setStyleSheet(
        tu.look_and_feel(
            app=app,
            font=font,
            adjust_width=adjust_width,
            adjust_height=adjust_height,
            default=content['Font']
            )
        )

    # Load content for the GUI
    content_gui = tu.get_content_gui(content=content)

    # Initilise and show GUI
    gui = MainWindow(
        content_gui=content_gui,
        content_pipeline=content['Pipeline'],
        settings_folder=settings_directory,
        mount_directory=mount_directory,
        version=transphire.__version__
        )
    gui.show()

    sys.exit(app.exec_())


def parse_args():
    """
    Handle command line arguments.

    Arguments:
    None

    Return:
    Dictionary of command line arguments
    """

    parser = argparse.ArgumentParser(description='Cryo-EM post-datacollection pipeline')
    parser.add_argument(
        '--root_directory',
        default=os.environ['HOME'],
        type=str,
        nargs='?',
        help='Directory where transphire is started'
        )
    parser.add_argument(
        '--settings_directory',
        default='.transphire_settings',
        type=str,
        nargs='?',
        help='Folder containing the settings'
        )
    parser.add_argument(
        '--mount_directory',
        default='mounted',
        type=str,
        nargs='?',
        help='Folder containing the mount points'
        )
    parser.add_argument(
        '--font',
        default=None,
        type=float,
        nargs='?',
        help='Font size in px (default 20)'
        )
    parser.add_argument(
        '--adjust_width',
        default=None,
        type=float,
        nargs='?',
        help='Width adjustment value (default 1)'
        )
    parser.add_argument(
        '--adjust_height',
        default=None,
        type=float,
        nargs='?',
        help='Height adjustment value (default 1)'
        )
    parser.add_argument(
        '--edit_settings',
        const='edit_settings',
        default=False,
        action='store_const',
        help='Show settings dialog (default False)'
        )

    return vars(parser.parse_args())


def run_package():
    """
    Entry point for the transphire package
    """
    main(**parse_args())


if __name__ == '__main__':
    run_package()



"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import sys
try:
    QT_VERSION = 4
    from PyQt4.QtGui import QWidget, QLabel, QFileDialog, QVBoxLayout, QComboBox, QLineEdit
    from PyQt4.QtCore import pyqtSlot, pyqtSignal
except ImportError:
    QT_VERSION = 5
    from PyQt5.QtWidgets import QWidget, QLabel, QFileDialog, QVBoxLayout, QComboBox, QLineEdit
    from PyQt5.QtCore import pyqtSlot, pyqtSignal
from transphire import transphire_utils as tu


class SettingsWidget(QWidget):
    """
    Widget for setting entrys

    Inherits:
    QWidget

    Signals:
    sig_index_changed - Emitted, if the index of a combo box changes (Combo box name|str)
    """
    sig_index_changed = pyqtSignal(str)

    def __init__(self, content, parent=None):
        """
        Initialise the layout.

        Arguments:
        content - Content for the widget.
        parent - Parent widget (default None)

        Returns:
        None
        """
        super(SettingsWidget, self).__init__(parent)

        # Layout
        layout = QVBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)

        # Global content
        self.default = content[0]
        self.typ = content[1]['typ']
        self.values = content[1]['values']
        self.name = content[1]['name']
        self.dtype = content[1]['dtype']

        if self.typ == 'PLAIN':
            self.edit = QLineEdit(self.name, self)
            self.edit.setText(self.default)

        elif self.typ == 'FILE':
            self.edit = QLineEdit(self.name, self)
            self.edit.setText(self.default)
            self.edit.setPlaceholderText('Press shift+return')
            self.edit.returnPressed.connect(self._find_file)

        elif self.typ == 'FILE/CHOICE':
            self.edit = QLineEdit(self.name, self)
            self.edit.setText(self.default)
            self.edit.setPlaceholderText('Press shift+return')
            self.edit.returnPressed.connect(self._find_file)

        elif self.typ == 'DIR':
            self.edit = QLineEdit(self.name, self)
            self.edit.setText(self.default)
            self.edit.setPlaceholderText('Press shift+return')
            self.edit.returnPressed.connect(self._find_dir)

        elif self.typ == 'COMBO':
            self.edit = QComboBox(self)
            self.edit.currentIndexChanged.connect(lambda: self.sig_index_changed.emit(self.name))
            self.edit.addItems(self.values)
            self.edit.setCurrentIndex(self.edit.findText(self.default))
            self.change_color_if_true()

        else:
            print('SettingsWidget:', self.typ, 'Not known! Stopping here!')
            sys.exit()

        self.label = QLabel(self.name, self)
        self.label.setObjectName('setting')
        self.edit.setObjectName('setting')
        try:
            self.edit.textEdited.connect(
                lambda: self.edit.setStyleSheet(tu.get_style(typ='unchanged'))
                )
        except AttributeError:
            pass
        try:
            self.edit.currentIndexChanged.connect(
                self.change_color_if_true
                )
        except AttributeError:
            pass
        layout.addWidget(self.label)
        layout.addWidget(self.edit)

    def change_color_if_true(self):
        """
        Change the color, if the types are all true.

        Arguments:
        None

        Returns:
        None
        """
        if self.edit.currentText() == 'False':
            self.edit.setStyleSheet(tu.get_style(typ='False'))
        else:
            self.edit.setStyleSheet(tu.get_style(typ='True'))

    @pyqtSlot()
    def _find_file(self):
        """
        Find file with an open file dialog.

        Arguments:
        None

        Returns:
        None
        """
        in_file = QFileDialog.getOpenFileName(
            caption='Find file: {0}'.format(self.name),
            directory=os.getcwd(),
            options=QFileDialog.DontUseNativeDialog
            )

        if QT_VERSION == 4:
            in_file = in_file
        elif QT_VERSION == 5:
            in_file = in_file[0]
        else:
            raise ImportError('QT version unknown! Please contact the transphire authors!')

        if in_file != '':
            self.sender().setText(in_file)

    @pyqtSlot()
    def _find_dir(self):
        """
        Find directory with an open directory dialog

        Arguments:
        None

        Returns:
        None
        """
        in_dir = QFileDialog.getExistingDirectory(
            caption='Find directory: {0}'.format(self.name),
            directory=os.getcwd(),
            options=QFileDialog.DontUseNativeDialog
            )
        if in_dir != '':
            self.sender().setText(in_dir)

    def get_settings(self, quiet=False):
        """
        Get the settings as dict.

        Arguments:
        quiet - True, if prints should not be shown.

        Returns:
        None, if an error occured.
        Settings as dictionary.
        """
        settings = {}
        if isinstance(self.edit, QComboBox):
            value = self.edit.currentText()

        elif isinstance(self.edit, QLineEdit):
            value = self.edit.text()

        else:
            message = 'Unreachable code! Please contact the TranSPHIRE authors!'
            print(message)
            tu.message(message)
            return None

        if value:

            if tu.check_instance(value=value, typ=self.dtype):
                self.edit.setStyleSheet(tu.get_style(typ='pass'))

            else:
                self.edit.setStyleSheet(tu.get_style(typ='error'))
                message = '{0}: {1} needs to be {2}'.format(
                    self.label.text(),
                    value,
                    self.dtype
                    )

                if not quiet:
                    print(message)
                    tu.message(message)

                else:
                    pass
                return None

        else:
            pass

        settings[self.name] = value

        return settings

    def get_combo_entries(self):
        """
        Get the entries of the combo boxes.

        Arguments:
        None

        Returns:
        List containing entries.
        """
        entries_list = []
        for idx in range(self.edit.count()):
            entries_list.append(self.edit.itemText(idx))
        return entries_list

    def set_settings(self, text):
        """
        Set settings

        text - Text to set to the widget.

        Returns:
        None
        """
        if self.typ == 'COMBO':
            index = self.edit.findText(text)
            if index == -1:
                index = 0
            else:
                pass
            self.edit.setCurrentIndex(index-1)
            self.edit.setCurrentIndex(index)
        else:
            self.edit.setText(text)



"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
try:
    from PyQt4.QtGui import QLabel
except ImportError:
    from PyQt5.QtWidgets import QLabel


class Separator(QLabel):
    """Separator widget"""

    def __init__(self, typ, color, parent=None):
        super(Separator, self).__init__(parent)

        self.setStyleSheet('background: {0}'.format(color))
        if typ == 'horizontal':
            self.setFixedHeight(2)
        elif typ == 'vertical':
            self.setFixedWidth(2)



"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import time
try:
    from PyQt4.QtCore import pyqtSignal, QObject, pyqtSlot
except ImportError:
    from PyQt5.QtCore import pyqtSignal, QObject, pyqtSlot
from transphire import transphire_utils as tu


class PlotWorker(QObject):
    """
    Plot different information about motion correction and ctf estimation.

    Inherits:
    QObject

    Signals:
    sig_data - Emitted, if data for plotting is found (name|str, data|object, directory|str, settings|object)
    sig_notification - Emitted, if phase plate limit is reached. (text|str)
    """
    sig_data = pyqtSignal(str, object, str, object)
    sig_notification = pyqtSignal(str)
    sig_message = pyqtSignal(str)

    def __init__(self, parent=None):
        """
        Initialise variables.

        Arguments:
        parent - Parent widget (default None)

        Returns:
        None
        """
        super(PlotWorker, self).__init__(parent)
        self.notified = False
        self.time_last_error = 0

    @pyqtSlot(str, object, object)
    def calculate_array_ctf(self, ctf_name, directory_name, settings):
        """
        Calculate ctf array.

        ctf_name - Name of the software that calls the calculation
        directory_name - Name of the directory that contains the log files
        settings - TranSPHIRE settings

        Returns:
        None
        """
        data, _ = tu.import_ctf(
            ctf_name=ctf_name,
            directory_name=directory_name
            )
        if data is None:
            return None
        elif data.size == 0:
            return None
        else:
            pass

        last_phase_shift = data['phase_shift'][-1]
        phase_shift_warning = float(settings['General']['Phase shift warning (deg)'])
        if last_phase_shift > phase_shift_warning:
            if self.notified:
                if last_phase_shift < 90:
                    self.notified = False
                else:
                    pass
            else:
                msg = '\n'.join([
                    '{0}:'.format(ctf_name),
                    'Phase shift is higher than {0}!'.format(phase_shift_warning),
                    'You might want to change the phase plate position.'
                    ])
                self.sig_notification.emit(msg)
                self.notified = True
                if time.time() - self.time_last_error > 1800:
                    self.sig_message.emit(msg)
                    self.time_last_error = time.time()
                else:
                    pass
        self.sig_data.emit(ctf_name, data, directory_name, settings)

    @pyqtSlot(str, object, object)
    def calculate_array_motion(self, motion_name, directory_name, settings):
        """
        Calculate motion array.

        motion_name - Name of the software that calls the calculation
        directory_name - Name of the directory that contains the log files
        settings - TranSPHIRE settings

        Returns:
        None
        """
        data = tu.import_motion(
            motion_name=motion_name,
            directory_name=directory_name
            )
        if data is None:
            return None
        elif data.size == 0:
            return None
        else:
            self.sig_data.emit(motion_name, data, directory_name, settings)



"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import glob
import copy as cp
import queue as qu
try:
    from PyQt4.QtCore import QObject, pyqtSignal, pyqtSlot, QThread, QMutex
except ImportError:
    from PyQt5.QtCore import QObject, pyqtSignal, pyqtSlot, QThread, QMutex
from transphire.processthread import ProcessThread
from transphire import transphire_utils as tu


class ProcessWorker(QObject):
    """
    Setup and start worker threads

    Inherits from:
    QObject

    Buttons:
    None

    Signals:
    sig_start - Connected to the run method to start the process (settings|object)
    sig_finished - Emitted, if run method finishes (No objects)
    sig_error - Emitted, if an error occured (text|str)
    sig_status - Emitted to change the status (text|str, device|str, color|str)
    sig_notification - Emitted to send a notification (text|str)
    sig_plot_ctf - Emitted to plot ctf information (ctf_name|str, ctf_settings|object, settings|object)
    sig_plot_motion - Emitted to plot motion information (motion_name|str, motion_settings|object, settings|object)
    """
    sig_start = pyqtSignal(object)
    sig_finished = pyqtSignal()
    sig_error = pyqtSignal(str)
    sig_status = pyqtSignal(str, str, str)
    sig_notification = pyqtSignal(str)
    sig_plot_ctf = pyqtSignal(str, object, object)
    sig_plot_motion = pyqtSignal(str, object, object)

    def __init__(self, password, content_process, mount_directory, parent=None):
        """
        Initialize object variables.

        Arguments:
        password - Sudo password
        content_process - Pipeline content
        mount_directory - Folder containing the mount points
        parent - Parent widget (default None)

        Return:
        None
        """
        super(ProcessWorker, self).__init__(parent)

        # Variables
        self.password = password
        self.content_process = content_process
        self.mount_directory = mount_directory
        self.stop = False
        self.settings = {}

        # Events
        self.sig_start.connect(self.run)

    @pyqtSlot(object)
    def run(self, settings):
        """
        Start the process.

        Arguments:
        settings - Transphire settings

        Return:
        None
        """
        # Set settings
        self.settings = settings
        content_process = cp.deepcopy(self.content_process)

        # Set stop variable to the return value of the pre_check
        self.stop = bool(self.pre_check_programs())

        # Set paths
        self.settings['stack_folder'] = os.path.join(
            self.settings['project_folder'], 'Stack'
            )
        self.settings['meta_folder'] = os.path.join(
            self.settings['project_folder'], 'Meta'
            )
        self.settings['software_meta_folder'] = os.path.join(
            self.settings['project_folder'], 'Software_meta'
            )
        self.settings['motion_folder'] = os.path.join(
            self.settings['motion_folder'],
            self.settings['Copy']['Motion'].replace(' ', '_')
            )
        self.settings['ctf_folder'] = os.path.join(
            self.settings['ctf_folder'],
            self.settings['Copy']['CTF'].replace(' ', '_')
            )
        self.settings['scratch_motion_folder'] = os.path.join(
            self.settings['scratch_folder'],
            self.settings['Copy']['Motion'].replace(' ', '_')
            )
        self.settings['compress_folder'] = os.path.join(
            self.settings['compress_folder'], 'Compress'
            )
        self.settings['Copy_hdd_folder'] = os.path.join(
            self.settings['Copy_hdd_folder'], 'HDD'
            )
        self.settings['Copy_work_folder'] = os.path.join(
            self.settings['Copy_work_folder'],
            self.settings['Copy']['Copy to work'].replace(' ', '_')
            )
        self.settings['Copy_backup_folder'] = os.path.join(
            self.settings['Copy_backup_folder'],
            self.settings['Copy']['Copy to backup'].replace(' ', '_')
            )

        # Set CTF settings
        self.settings['CTF_folder'] = {}
        for entry in self.settings['Copy']['CTF_entries']:
            self.settings['CTF_folder'][entry] = os.path.join(
                self.settings['project_folder'],
                entry.replace(' ', '_')
                )
            self.sig_plot_ctf.emit(
                entry,
                self.settings['CTF_folder'][entry],
                self.settings
                )

        # Set Motion settings
        self.settings['Motion_folder'] = {}
        for entry in self.settings['Copy']['Motion_entries']:
            self.settings['Motion_folder'][entry] = os.path.join(
                self.settings['project_folder'],
                entry.replace(' ', '_')
                )
            self.sig_plot_motion.emit(
                entry,
                self.settings['Motion_folder'][entry],
                self.settings
                )

        # Set default values for folder list and thread list
        folder_list = ['stack_folder', 'meta_folder']
        use_threads_list = ['Meta', 'Find', 'Copy']

        # Fill folder list and threads list
        for name in ['work', 'backup']:
            short_name = 'Copy_{0}'.format(name)
            long_name = 'Copy to {0}'.format(name)
            folder_name = '{0}_folder'.format(short_name)
            user_name = '{0}_user'.format(short_name)
            self.settings['Copy'][short_name] = \
                self.settings['Copy'][long_name]

            if self.settings['Copy'][long_name] != 'False':
                if self.settings['Copy'][long_name] == 'Later':
                    pass
                elif not os.path.ismount(self.settings[folder_name]):
                    try:
                        os.listdir(self.settings[folder_name])
                    except PermissionError:
                        pass
                    except FileNotFoundError:
                        self.sig_error.emit(
                            '{0} folder {1} not mounted!'.format(
                                name,
                                self.settings['Copy'][long_name]
                                )
                            )
                        self.sig_finished.emit()
                        return None
                    except OSError as err:
                        if 'Required key' in str(err):
                            self.sig_error.emit(
                                '\n'.join([
                                    '{0} folder {1} no longer mounted! '.format(
                                        name,
                                        self.settings['Copy'][long_name]
                                        ),
                                    'Use kinit to refresh the key'
                                    ])
                                )
                            self.sig_finished.emit()
                            return None
                        else:
                            print(
                                '\n'.join([
                                    'Unknown Error occured!',
                                    'Please contact the TranSPHIRE authors!'
                                    ])
                            )
                            raise
                    else:
                        self.sig_error.emit(
                            '{0} folder {1} not mounted!'.format(
                                name,
                                self.settings['Copy'][long_name]
                                )
                            )
                        self.sig_finished.emit()
                        return None

                else:
                    pass
                try:
                    self.settings[user_name] = self.settings[
                        'user_{0}'.format(
                            self.settings['Copy'][long_name].replace(' ', '_')
                            )
                        ]
                except KeyError:
                    self.sig_error.emit('{0} needs remount! '.format(name))
                    self.sig_finished.emit()
                    return None
                use_threads_list.append(short_name)
            else:
                pass

        # Decide if one will use copy to HDD
        self.settings['Copy']['Copy_hdd'] = self.settings['Copy']['Copy to HDD']
        if self.settings['Copy']['Copy to HDD'] != 'False':
            if self.settings['Copy']['Copy to HDD'] == 'Later':
                pass
            else:
                try:
                    for folder in glob.glob('{0}/*'.format(self.settings['Copy_hdd_folder'])):
                        if not os.path.ismount(folder):
                            try:
                                os.listdir(folder)
                            except PermissionError:
                                pass
                            else:
                                self.sig_error.emit(
                                    'HDD not mounted or not well unmounted!' +
                                    'Please remount if you want to use HDD'
                                    )
                                self.sig_finished.emit()
                                return None
                        else:
                            pass
                except FileNotFoundError:
                    self.sig_error.emit(
                        'HDD not mounted or not well unmounted!' +
                        'Please remount if you want to use HDD!'
                        )
                    self.sig_finished.emit()
                    return None
            use_threads_list.append('Copy_hdd')
        else:
            pass

        # Set CTF settings
        if self.settings['Copy']['CTF'] != 'False':
            folder_list.append('ctf_folder')
            use_threads_list.append('CTF')
            ctf_name = self.settings['Copy']['CTF']
            try:
                if self.settings[ctf_name]['Use movies'] == 'True':
                    self.settings['Copy']['CTF_frames'] = 'True'
                    self.settings['Copy']['CTF_sum'] = 'False'
                else:
                    self.settings['Copy']['CTF_frames'] = 'False'
                    self.settings['Copy']['CTF_sum'] = 'True'
            except KeyError:
                self.settings['Copy']['CTF_frames'] = 'False'
                self.settings['Copy']['CTF_sum'] = 'True'
        else:
            self.settings['Copy']['CTF_frames'] = 'False'
            self.settings['Copy']['CTF_sum'] = 'False'

        # Set Compress settings
        if self.settings['Copy']['Compress data'] != 'False':
            folder_list.append('compress_folder')
            use_threads_list.append('Compress')
        else:
            pass

        # Set Motion settings
        if self.settings['Copy']['Motion'] != 'False':
            folder_list.append('motion_folder')
            use_threads_list.append('Motion')
        else:
            pass

        # Create output directories
        for entry in folder_list:
            try:
                tu.mkdir_p(self.settings[entry])
            except PermissionError:
                continue
            except OSError as err:
                print(str(err))
                self.sig_error.emit(str(err))
                self.sig_finished.emit()
                return None

        # Queue communication dictionary
        queue_com = {
            'status': qu.Queue(),
            'notification': qu.Queue(),
            'error': qu.Queue(),
            'plot_ctf': qu.Queue(),
            'plot_motion': qu.Queue()
            }

        # Fill different dictionarys with process information
        typ_dict = {}
        wait_dict = {}
        share_dict = {}
        bad_dict = {}
        queue_dict = {}
        full_content = []
        idx_number = 0
        idx_values = 1
        for entry in content_process:
            for process in entry:
                for key in process:
                    process[key][idx_values]['group'], process[key][idx_values]['aim'] = \
                        process[key][idx_values]['group'].split(';')
                    process[key][idx_values]['aim'] = process[key][idx_values]['aim'].split(',')
                    wait_dict[key] = False
                    share_dict[key] = []
                    bad_dict[key] = []
                    queue_dict[key] = qu.Queue()
                    typ_dict[key] = {
                        'file_number': 0,
                        'spot': False,
                        'lost_input_meta': False,
                        'lost_input_frames': False,
                        'lost_work': False,
                        'lost_backup': False,
                        'lost_hdd': False,
                        'full_transphire': False,
                        'full_work': False,
                        'full_backup': False,
                        'full_hdd': False,
                        'unknown_error': False,
                        'queue_lock': QMutex(),
                        'save_lock': QMutex(),
                        'count_lock': QMutex(),
                        'error_lock': QMutex(),
                        'bad_lock': QMutex(),
                        'share_lock': QMutex(),
                        'spot_dict': self.fill_spot_dict(),
                        'number_file': '{0}/last_filenumber.txt'.format(
                            self.settings['project_folder']
                            ),
                        'save_file': '{0}/Queue_{1}'.format(
                            self.settings['queue_folder'], key
                            ),
                        'done_file': '{0}/Queue_{1}_done'.format(
                            self.settings['queue_folder'], key
                            ),
                        'error_file': '{0}/Queue_{1}_error'.format(
                            self.settings['error_folder'], key
                            ),
                        }

                    if process[key][idx_number] == '1':
                        full_content.append([key, process[key]])
                    else:
                        for idx in range(int(process[key][idx_number])):
                            full_content.append(
                                ['{0}_{1}'.format(key, idx+1), process[key]]
                                )

        # Shared dictionary
        shared_dict = {
            'share': share_dict,
            'bad': bad_dict,
            'queue': queue_dict,
            'translate_lock': QMutex(),
            'ctf_star_lock': QMutex(),
            'ctf_partres_lock': QMutex(),
            'global_lock': QMutex(),
            'typ': typ_dict
            }

        # Fill process queues
        for entry in content_process:
            for process in entry:
                for key in process:
                    self.prefill_queue(
                        shared_dict=shared_dict,
                        entry=process[key][1]
                        )

        # Define file number and check if file already exists
        shared_dict['typ']['Copy']['file_number'] = int(self.settings['General']['Start number']) - 1

        if self.settings['General']['Rename micrographs'] == 'True':
            new_name = '{0}/{1}{2:0{3}d}{4}'.format(
                self.settings['meta_folder'],
                self.settings['General']['Rename prefix'],
                shared_dict['typ']['Copy']['file_number']+1,
                len(self.settings['General']['Estimated mic number']),
                self.settings['General']['Rename suffix']
                )

            if os.path.exists('{0}.jpg'.format(new_name)):
                self.stop = True
                try:
                    with open(shared_dict['typ']['Copy']['number_file'], 'r') as read:
                        shared_dict['typ']['Copy']['file_number'] = int(read.readline())
                except FileNotFoundError:
                    shared_dict['typ']['Copy']['file_number'] = int(self.settings['General']['Start number'])
                message = '{0}: File {1} already exists!\n'.format(
                    'Copy',
                    new_name
                    ) + \
                    'Check Startnumber!'
                queue_com['error'].put(message)
                queue_com['notification'].put(message)
            else:
                pass
        else:
            pass

        # Start threads
        thread_list = []
        use_threads_set = set(use_threads_list)
        for key, settings_content in full_content:
            content_settings = settings_content[idx_values]
            thread = ProcessThread(
                shared_dict=shared_dict,
                name=key,
                content_settings=content_settings,
                queue_com=queue_com,
                settings=self.settings,
                mount_directory=self.mount_directory,
                password=self.password,
                use_threads_set=use_threads_set,
                stop=self.stop,
                parent=self
                )
            thread.start()
            thread_list.append([thread, key, content_settings])
            QThread.sleep(1)

        # Run until the user stops the processes
        go_on = True
        while go_on:
            try:
                self.check_queue(queue_com=queue_com)
            except BrokenPipeError:
                pass
            if self.stop:
                go_on = False
            else:
                pass
            QThread.sleep(3)

        # Indicate to stop all processes
        for key, settings_content in full_content:
            typ = settings_content[idx_values]['name']
            size = shared_dict['queue'][typ].qsize()
            self.sig_status.emit(
                'Stopping {0} {1}'.format(
                    size,
                    shared_dict['typ'][typ]['file_number']
                    ),
                key,
                'red'
                )

        # Send the stop signals to all threads
        for thread, name, setting in thread_list:
            thread.stop = True

        # Wait for all threads to finish
        for thread, name, setting in thread_list:
            typ = setting['name']
            thread.quit()
            thread.wait()
            self.check_queue(queue_com=queue_com)
            size = shared_dict['queue'][typ].qsize()
            self.sig_status.emit(
                'Not running {0} {1}'.format(
                    size,
                    shared_dict['typ'][typ]['file_number']
                    ),
                name,
                'purple'
                )

        self.sig_finished.emit()

    def pre_check_programs(self):
        """
        Check, if all programs the user wants to use are available.

        Arguments:
        None

        Return:
        True, if programs exist, else False
        """
        error = False
        check_files = []
        check_files.append(self.settings['Copy']['Motion'])
        check_files.append(self.settings['Copy']['CTF'])
        check_files.append('IMOD header')
        if len(self.settings['motion_frames']) > 1:
            check_files.append('SumMovie v1.0.2')
        for name in check_files:
            if name != 'False' and name != 'Later':
                if not os.path.isfile(self.settings['Path'][name]):
                    self.sig_error.emit(
                        '{0} path not valid! Please adjust it!'.format(name)
                        )
                    error = True
                else:
                    pass
            else:
                pass

        return error

    def check_queue(self, queue_com):
        """
        Check the content of the queues and react accordingly.

        Arguments:
        queue_com

        Return:
        None
        """
        for key in queue_com:
            while not queue_com[key].empty():
                if key == 'status':
                    status, device, color = queue_com['status'].get()
                    self.sig_status.emit(status, device, color)
                elif key == 'notification':
                    notification = queue_com['notification'].get()
                    self.sig_notification.emit(notification)
                elif key == 'error':
                    error = queue_com['error'].get()
                    self.sig_error.emit(error)
                elif key == 'plot_ctf':
                    queue_com['plot_ctf'].get()
                    self.sig_plot_ctf.emit(
                        self.settings['Copy']['CTF'],
                        self.settings['CTF_folder'][
                            self.settings['Copy']['CTF']
                            ],
                        self.settings
                        )
                elif key == 'plot_motion':
                    queue_com['plot_motion'].get()
                    self.sig_plot_motion.emit(
                        self.settings['Copy']['Motion'],
                        self.settings['Motion_folder'][
                            self.settings['Copy']['Motion']
                            ],
                        self.settings
                        )
                else:
                    print(
                        'Processworker - check_queue:',
                        key,
                        ' Not known!',
                        ' Unreachable code!',
                        ' Please contact the TranSPHIRE authors'
                        )

    def fill_spot_dict(self):
        """
        Fill the spot dictionary.

        Arguments:
        None

        Return:
        Spot dictionary
        """
        dictionary = {}
        spot_file = os.path.join(self.settings['project_folder'], '.spot_dict')
        try:
            with open(spot_file, 'r') as read:
                lines = [line.rstrip() for line in read.readlines()]
            for line in lines:
                key, number = line.split('\t')
                dictionary[key] = number
        except FileNotFoundError:
            with open(spot_file, 'w') as write:
                write.write('')
        return dictionary

    def prefill_queue(self, shared_dict, entry):
        """
        Prefill the queues for continue mode

        Arguments:
        shared_dict - Shared dictionary
        entry - Name of the queue process

        Return:
        None
        """
        key = entry['name']
        share = entry['group']
        shared_dict_typ = shared_dict['typ'][key]
        save_file = shared_dict_typ['save_file']
        done_file = shared_dict_typ['done_file']
        share_list = shared_dict['share'][share]
        queue = shared_dict['queue'][key]

        if os.path.exists(save_file):
            with open(save_file, 'r') as read:
                lines = [line.rstrip() for line in read.readlines()]

            for line in lines:
                if self.settings['Copy_software_meta']:
                    # Dont fill queue for Meta files
                    if self.settings['software_meta_folder'] in line:
                        continue
                    else:
                        pass
                else:
                    pass
                if line.startswith(self.settings['project_folder']):
                    share_list.append(line)
                    queue.put(line)
                else:
                    pass
        else:
            with open(save_file, 'w'):
                pass

        if os.path.exists(done_file):
            with open(done_file, 'r') as read:
                shared_dict_typ['file_number'] = len(read.readlines())
        else:
            with open(done_file, 'w'):
                pass
